;==============================================================================
    include \masm32\include\masm32rt.inc
;==============================================================================

printf MACRO format:REQ, args:VARARG
    IFNB <args>
        invoke crt_printf, cfm$(format), args
    ELSE
        invoke crt_printf, cfm$(format)
    ENDIF
    EXITM <>
ENDM

;==============================================================================
    .data
    .code
;==============================================================================

;------------------------------------------------------------------
; This procedure creates a DIB from a DDB and stores it in a file,
; or silently fails if the bits per pixel (bpp) for the DDB is not
; 4, 8, 16, 24, or 32, or if the bpp value for the DIB, specified
; in bppDib, is not 4, 8, or 24.
;------------------------------------------------------------------

CreateBitmapFile proc uses ebx edi hbmp:HBITMAP,
                                   bppDib:DWORD,
                                   pszFile:DWORD

    LOCAL dwtmp:DWORD
    LOCAL hFile:HANDLE
    LOCAL hHeap:HANDLE
    LOCAL hdcScreen:HDC
    LOCAL hdr:BITMAPFILEHEADER

    invoke GetProcessHeap
    mov hHeap, eax

    invoke GetDC, NULL
    mov hdcScreen, eax

    invoke HeapAlloc, hHeap, HEAP_ZERO_MEMORY, SIZEOF BITMAPINFO
    mov ebx, eax

    mov [ebx].BITMAPINFO.bmiHeader.biSize, SIZEOF BITMAPINFOHEADER

    invoke GetDIBits, hdcScreen, hbmp, 0, 0, NULL,
                      ebx, DIB_RGB_COLORS

    movzx eax, [ebx].BITMAPINFO.bmiHeader.biBitCount
    SWITCH eax
      CASE 4, 8, 16, 24, 32
      DEFAULT
        invoke HeapFree, hHeap, 0, ebx
        ret
    ENDSW

    mov eax, bppDib
    SWITCH eax
      CASE 4, 8, 24
        mov [ebx].BITMAPINFO.bmiHeader.biBitCount, ax
      DEFAULT
        invoke HeapFree, hHeap, 0, ebx
        ret
    ENDSW

    movzx eax, [ebx].BITMAPINFO.bmiHeader.biBitCount
    SWITCH eax
      CASE 4
        invoke HeapReAlloc, hHeap, HEAP_ZERO_MEMORY, ebx,
                            SIZEOF BITMAPINFO + 16 * SIZEOF RGBQUAD
        mov ebx, eax
      CASE 8
        invoke HeapReAlloc, hHeap, HEAP_ZERO_MEMORY, ebx,
                            SIZEOF BITMAPINFO + 256 * SIZEOF RGBQUAD
        mov ebx, eax
    ENDSW

    mov [ebx].BITMAPINFO.bmiHeader.biCompression, BI_RGB

    movzx edi, [ebx].BITMAPINFO.bmiHeader.biBitCount
    mov eax, [ebx].BITMAPINFO.bmiHeader.biWidth
    mul edi
    add eax, 31
    and eax, NOT 31
    shr eax, 3
    mov ecx, [ebx].BITMAPINFO.bmiHeader.biHeight
    mul ecx
    mov [ebx].BITMAPINFO.bmiHeader.biSizeImage, eax

    invoke GetDeviceCaps, hdcScreen, LOGPIXELSX
    invoke MulDiv, eax, 39370, 1000
    mov [ebx].BITMAPINFO.bmiHeader.biXPelsPerMeter, eax

    invoke GetDeviceCaps, hdcScreen, LOGPIXELSY
    invoke MulDiv, eax, 39370, 1000
    mov [ebx].BITMAPINFO.bmiHeader.biYPelsPerMeter, eax

    movzx eax, [ebx].BITMAPINFO.bmiHeader.biBitCount
    .IF eax == 4
      mov [ebx].BITMAPINFO.bmiHeader.biClrUsed, 16
    .ELSEIF eax == 8
      mov [ebx].BITMAPINFO.bmiHeader.biClrUsed, 256
    .ENDIF

    mov [ebx].BITMAPINFO.bmiHeader.biClrImportant, 0

    invoke HeapAlloc, hHeap, HEAP_ZERO_MEMORY,
                      [ebx].BITMAPINFO.bmiHeader.biSizeImage
    mov edi, eax

    push [ebx].BITMAPINFO.bmiHeader.biClrUsed
    invoke GetDIBits, hdcScreen,
                      hbmp,
                      0,
                      [ebx].BITMAPINFO.bmiHeader.biHeight,
                      edi,
                      ebx,
                      DIB_RGB_COLORS
    pop [ebx].BITMAPINFO.bmiHeader.biClrUsed

    invoke CreateFile, pszFile,
                       GENERIC_READ or GENERIC_WRITE,
                       0,
                       NULL,
                       CREATE_ALWAYS,
                       FILE_ATTRIBUTE_NORMAL,
                       NULL
    mov hFile, eax

    mov eax, [ebx].BITMAPINFO.bmiHeader.biClrUsed
    mov ecx, SIZEOF RGBQUAD
    mul ecx
    add eax, SIZEOF BITMAPFILEHEADER
    add eax, [ebx].BITMAPINFO.bmiHeader.biSize
    push eax
    add eax, [ebx].BITMAPINFO.bmiHeader.biSizeImage

    mov hdr.bfType, 4d42h

    mov hdr.bfSize, eax
    mov hdr.bfReserved1, 0
    mov hdr.bfReserved2, 0
    pop eax
    mov hdr.bfOffBits, eax

    invoke WriteFile, hFile,
                      ADDR hdr,
                      SIZEOF BITMAPFILEHEADER,
                      ADDR dwtmp,
                      NULL

    mov eax, [ebx].BITMAPINFO.bmiHeader.biClrUsed
    mov ecx, SIZEOF RGBQUAD
    mul ecx
    add eax, SIZEOF BITMAPINFOHEADER
    mov edx, eax

    invoke WriteFile, hFile,
                      ebx,
                      edx,
                      ADDR dwtmp,
                      NULL

    invoke WriteFile, hFile,
                      edi,
                      [ebx].BITMAPINFO.bmiHeader.biSizeImage,
                      ADDR dwtmp,
                      NULL

    invoke ReleaseDC, NULL, hdcScreen
    invoke HeapFree, hHeap, 0, ebx
    invoke HeapFree, hHeap, 0, edi
    invoke CloseHandle, hFile

    ret

CreateBitmapFile endp

;==============================================================================

DlgProc proc hwndDlg:dword, uMsg:dword, wParam:dword, lParam:dword

    LOCAL hdc:HDC
    LOCAL hBitmap:HGDIOBJ

    SWITCH uMsg

      CASE WM_INITDIALOG

      CASE WM_COMMAND

        SWITCH wParam

          CASE 1000

            invoke GetDC, hwndDlg
            mov hdc, eax
            printf( "hdc = %Xh\n", hdc )

            invoke GetCurrentObject, hdc, OBJ_BITMAP
            mov hBitmap, eax
            printf( "hBitmap = %Xh\n", hBitmap )

            invoke CreateBitmapFile, hBitmap, 24, chr$("cap.bmp")

          CASE IDCANCEL

            invoke EndDialog, hwndDlg, 0

        ENDSW

      CASE WM_CLOSE

        invoke EndDialog, hwndDlg, 0

    ENDSW

    xor eax, eax
    ret

DlgProc endp

;==============================================================================
start:
;==============================================================================

    Dialog "Test", "MS Sans Serif",10, \
           WS_OVERLAPPED or WS_SYSMENU or DS_CENTER, \
           1,0,0,100,75,1024

    DlgButton "Capture",0,34,40,30,12,1000

    invoke GetModuleHandle, NULL

    CallModalDialog eax,0,DlgProc,NULL

    exit
;==============================================================================
end start